/*  fgwirc - fungw based minimal irc client
    Copyright (C) 2019  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page and contact: http://repo.hu/projects/fgwirc
*/

#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <genht/htsp.h>
#include <genht/hash.h>
#include <libfungw/fungw.h>
#include "conf.h"

static htsp_t conf;

void conf_init(void)
{
	htsp_init(&conf, strhash, strkeyeq);
}

void conf_uninit(void)
{
	htsp_entry_t *e;
	for(e = htsp_first(&conf); e != NULL; e = htsp_next(&conf, e)) {
		conf_item_t *c = e->value;
		free(c->name);
		free(c->desc);
		free(c);
	}
	htsp_uninit(&conf);
}


void conf_reg(const char *name, fgw_type_t type, void *var, void (*on_chg)(conf_item_t *item), const char *desc)
{
	conf_item_t *c = malloc(sizeof(conf_item_t));
	assert(htsp_get(&conf, name) == NULL);
	c->name = fgw_strdup(name);
	c->type = type;
	c->var = var;
	c->on_chg = on_chg;
	c->desc = fgw_strdup(desc);
	c->orig = 1;
	htsp_set(&conf, c->name, c);
	assert(type == FGW_STR || type == FGW_INT);
}

int conf_set(char *name, char *val)
{
	char *end;
	conf_item_t *c = htsp_get(&conf, name);

	if (c == NULL)
		return -1;

	switch(c->type) {
		case FGW_STR:
			{
				char **s = c->var;
				if (!c->orig)
					free(*s);
				*s = fgw_strdup(val);
				c->orig = 0;
				break;
			}

		case FGW_INT:
			{
				int *v, itmp = strtol(val, &end, 10);
				if (*end != '\0')
					return -1;
				v = c->var;
				*v = itmp;
				c->orig = 0;
				break;
			}
		default:
			return -1;
	}

	if (c->on_chg != NULL)
		c->on_chg(c);

	return 0;
}

conf_item_t *conf_get(char *name)
{
	return htsp_get(&conf, name);
}


